@@ -3,7 +3,7 @@ our $AUTHORITY = 'cpan:GENE';
 
 # ABSTRACT: Generate walking basslines
 
-our $VERSION = '0.0512';
+our $VERSION = '0.0600';
 
 use Moo;
 use strictures 2;
@@ -104,6 +104,11 @@ has tonic => (
 );
 
 
+has positions => (
+    is => 'ro',
+);
+
+
 has verbose => (
     is      => 'ro',
     isa     => \&_boolean,
@@ -142,7 +147,20 @@ sub generate {
     my @notes = map { $self->pitchnum($_) }
         $cn->chord_with_octave($chord, $self->octave);
 
-    my @pitches = $scale ? get_scale_MIDI($chord_note, $self->octave, $scale) : ();
+    my @pitches;
+    if ($self->positions && $scale) {
+        my @scale = get_scale_MIDI($chord_note, $self->octave, $scale);
+        for my $n (0 .. $#scale) {
+            push @pitches, $scale[$n] if grep { $_ == $n } $self->positions->{$scale}->@*;
+        }
+    }
+    elsif ($scale) {
+        @pitches = get_scale_MIDI($chord_note, $self->octave, $scale);
+    }
+    else {
+        @pitches = ();
+    }
+
     my @next_pitches = $next_scale ? get_scale_MIDI($next_chord_note, $self->octave, $next_scale) : ();
 
     # Add unique chord notes to the pitches
@@ -207,16 +225,22 @@ sub generate {
     @fixed = uniq @fixed;
     $self->_verbose_notes('NOTES', @fixed) if $self->verbose;
 
-    my $voice = Music::VoiceGen->new(
-        pitches   => \@fixed,
-        intervals => $self->intervals,
-    );
+    my @chosen;
+    if (@fixed > 1) {
+        my $voice = Music::VoiceGen->new(
+            pitches   => \@fixed,
+            intervals => $self->intervals,
+        );
 
-    # Try to start the phrase in the middle of the scale
-    $voice->context($fixed[int @fixed / 2]);
+        # Try to start the phrase in the middle of the scale
+        $voice->context($fixed[int @fixed / 2]);
 
-    # Get a passage of quasi-random pitches
-    my @chosen = map { $voice->rand } 1 .. $num;
+        # Get a passage of quasi-random pitches
+        @chosen = map { $voice->rand } 1 .. $num;
+    }
+    else {
+        @chosen = ($fixed[0]) x $num;
+    }
 
     # Choose the right note given the scale if the tonic is set
     if ($self->tonic) {
@@ -294,7 +318,7 @@ MIDI::Bassline::Walk - Generate walking basslines
 
 =head1 VERSION
 
-version 0.0512
+version 0.0600
 
 =head1 SYNOPSIS
 
@@ -430,6 +454,16 @@ phrase.
 
 Default: C<0>
 
+=head2 positions
+
+  $positions = $bassline->positions;
+
+Optional lists of allowed notes for major and minor scales.
+
+Example: C<{ major =E<gt> [0..6], minor =E<gt> [0..6] }>
+
+Default: C<undef>
+
 =head2 verbose
 
   $verbose = $bassline->verbose;
@@ -446,6 +480,7 @@ Default: C<0>
 
   $bassline = MIDI::Bassline::Walk->new;
   $bassline = MIDI::Bassline::Walk->new(
+      positions   => $positions,
       guitar      => $guitar,
       intervals   => $intervals,
       octave      => $octave,
@@ -453,6 +488,7 @@ Default: C<0>
       chord_notes => $chord_notes,
       modal       => $modal,
       keycenter   => $key_center,
+      tonic       => $tonic,
       verbose     => $verbose,
   );
 
